'''
Author : hupeng
Time : 2021/8/9 14:44 
Description: 
''' 
class ParametersError(ValueError):
    '''参数错误'''
    pass


class RegularError(BaseException):
    '''正则匹配错误'''
    pass


class SimilarityError(BaseException):
    '''相似度错误'''
    pass


class ServerError(BaseException):
    pass


class TimeoutError(Exception):
    pass


class ApiError(Exception):
    pass



'''
Author : hupeng
Time : 2021/1/12 10:55 
Description: 
'''
import os
import json
import logging
from functools import partial
from logging.handlers import TimedRotatingFileHandler

import flask
from flask import g


class RequestId():
    def __init__(self):
        self.requestid = None


rd = RequestId()


class Record():
    def __init__(self):
        self.d = {}

    def reset(self):
        self.d.clear()

    def is_empty(self):
        return False if self.d else True

    def dumps(self):
        return json.dumps(self.d)

    def __setitem__(self, key, value):
        self.d[key] = value

    def __getitem__(self, item):
        return self.d[item]

    def __str__(self):
        return str(self.d)

    def __repr__(self):
        return str(self.d)


processing = Record()
time_record = Record()
statistics_record = Record()


class Formatter(logging.Formatter):
    def format(self, record):
        if flask.globals._request_ctx_stack.top is not None:
            record.requestid = g.requestid
        else:
            record.requestid = rd.requestid
        result = super(Formatter, self).format(record)
        return result


class MyLogging():

    def __init__(self, log_dir, log_name='', file_name='error'):
        self.logger = logging.getLogger(log_name)
        self.base_dir = log_dir
        self.check_root_dir()
        self.set_log_level(file_name)
        self.formatter = Formatter(
            f'[%(asctime)s] |%(levelname)s|[line:%(lineno)d][requestid:%(requestid)s] %(message)s')
        filename = os.path.join(self.base_dir, '{}_{}.log'.format(file_name, os.getpid()))
        file_handler = TimedRotatingFileHandler(filename=filename, when='MIDNIGHT')
        file_handler.setFormatter(self.formatter)
        self.logger.addHandler(file_handler)

    def set_log_level(self, log_level):
        if log_level == 'error':
            self.logger.setLevel(logging.ERROR)
        elif log_level == 'debug':
            self.logger.setLevel(logging.DEBUG)
        else:
            self.logger.setLevel(logging.INFO)

    def check_root_dir(self):
        path = f'{self.base_dir}'
        if not os.path.exists(path):
            os.makedirs(path)

    def processing(self, msg):
        self.write('[processing] %s' % str(msg))

    def write(self, msg):
        if not isinstance(msg, str):
            try:
                msg = str(msg)
            except:
                msg = msg
        if self.logger.level == logging.ERROR:
            self.logger.error(msg)
        elif self.logger.level == logging.DEBUG:
            self.logger.debug(msg)
        else:
            self.logger.info(msg)


error_log = partial(MyLogging, log_name='error', file_name='error')
# server_log = MyLogging(log_name='server', file_name='server')
# time_log = MyLogging(log_name='time', file_name='time')
# statistics_log = MyLogging(log_name='statistics', file_name='statistics')



'''
Author : hupeng
Time : 2021/8/6 14:47 
Description: 
'''
import json
import traceback

from flask import request

from fairy.const import Code
from fairy.utils.log import error_log
from fairy.utils.response import MyResponse


class BaseConfig(object):
    def __init__(self):
        self.data = {}

    def dumps(self):
        return json.dumps(self.data)

    def __setitem__(self, key, value):
        self.data[key] = value

    def __getitem__(self, item):
        return self.data[item]

    def __str__(self):
        return str(self.data)

    def __repr__(self):
        return str(self.data)


class BaseService(object):
    config = None
    params_class = None
    __is_init = False

    @classmethod
    def init(cls, cfg: dict):
        print('init...')
        cls.error_log = cfg.get('log_dir', error_log('./log'))
        cls.CODE_SUCCESS = cfg.get('CODE_SUCCESS', Code(0, '成功'))
        cls.CODE_PARAM = cfg.get('CODE_PARAM', Code(10000, '参数错误'))
        cls.CODE_SERVICE = cfg.get('CODE_SERVICE', Code(10001, '服务异常'))
        cls.__is_init = True

    def __init__(self):
        self._request = request
        self.params = None

    def get_params(self):
        if self.params_class is None:
            return
        params = self.params_class.build()
        self.params = params
        return params

    def prints(self, text: str, p=True, c=False):
        if p and c:
            print('\033[0;32m{}\033[0m'.format(text))
        elif p:
            print(text)

    def make_response(self) -> MyResponse:
        response = MyResponse()
        # 参数校验
        try:
            if not self.__is_init:
                self.init({})
            self.get_params()
        except Exception as e:
            response.code = self.CODE_PARAM.val
            response.message = str(e)
            self.error_log.write('got a params error %s' %
                            str(traceback.format_exc()))
            return response

        response.code = self.CODE_SUCCESS.val
        response.message = self.CODE_SUCCESS.msg
        try:
            response = self.get_response(response)
        except Exception as e:
            response.code = self.CODE_SERVICE.val
            response.message = f'{self.CODE_SERVICE.msg}: {str(e)}'
            self.error_log.write(str(traceback.format_exc()))
        return response

    def get_response(self, response: MyResponse) -> MyResponse:
        raise NotImplementedError()



'''
Author : hupeng
Time : 2021/8/6 14:49 
Description: 
'''
import json

from flask import Response


class MyResponse(object):

    def __init__(self, code: int = None, message: str = None, data: dict = None):
        self.code = code or None
        self.message = message or ''
        self.data = data or {}

    @property
    def dict(self):
        return self.__dict__

    def dumps(self):
        return json.dumps(self.dict)


def responser(response: MyResponse = None, *, code=0, data=None, message='') -> Response:
    if response is None:
        response = MyResponse()
        response.code = code
        response.message = str(message)
        response.data = data
    return Response(
        response=json.dumps(response.dict, ensure_ascii=False),
        mimetype='application/json',
    )
